"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NestjsQueryGraphQLModule = void 0;
const nestjs_query_core_1 = require("@rezonate/nestjs-query-core");
const providers_1 = require("./providers");
const subscription_1 = require("./subscription");
const types_1 = require("./types");
class NestjsQueryGraphQLModule {
    static forFeature(opts) {
        const coreModule = this.getCoreModule(opts);
        const providers = this.getProviders(opts);
        return {
            module: NestjsQueryGraphQLModule,
            imports: [...opts.imports, coreModule],
            providers: [...providers],
            exports: [...providers, ...opts.imports, coreModule]
        };
    }
    static defaultPubSubProvider() {
        return { provide: (0, subscription_1.pubSubToken)(), useValue: (0, subscription_1.defaultPubSub)() };
    }
    static getCoreModule(opts) {
        return nestjs_query_core_1.NestjsQueryCoreModule.forFeature({
            assemblers: opts.assemblers,
            imports: opts.imports
        });
    }
    static getProviders(opts) {
        return [
            ...this.getServicesProviders(opts),
            ...this.getPubSubProviders(opts),
            ...this.getAuthorizerProviders(opts),
            ...this.getHookProviders(opts),
            ...this.getResolverProviders(opts),
            ...this.getScalarsProviders()
        ];
    }
    static getScalarsProviders() {
        return [types_1.ConnectionCursorScalar];
    }
    static getPubSubProviders(opts) {
        return [opts.pubSub ?? this.defaultPubSubProvider()];
    }
    static getServicesProviders(opts) {
        return opts.services ?? [];
    }
    static getResolverProviders(opts) {
        return (0, providers_1.createResolvers)(opts.resolvers ?? []);
    }
    static getAuthorizerProviders(opts) {
        const resolverDTOs = opts.resolvers?.map((r) => r.DTOClass) ?? [];
        const dtos = opts.dtos?.map((o) => o.DTOClass) ?? [];
        return (0, providers_1.createAuthorizerProviders)([...resolverDTOs, ...dtos]);
    }
    static getHookProviders(opts) {
        return (0, providers_1.createHookProviders)([...(opts.resolvers ?? []), ...(opts.dtos ?? [])]);
    }
}
exports.NestjsQueryGraphQLModule = NestjsQueryGraphQLModule;
//# sourceMappingURL=module.js.map